// Author: Juraj Vanek
// This shader performs vertical blur
out vec4 out_FragColor;
in vec2 fragTexCoord;

uniform sampler2D bloomTexture;
uniform int kernel_size;
uniform float blur_radius;

//Gaussian filter constant
const float sigma = 25.132741;  // = 2*PI*sigma^2
const float frac_sqrt_sigma =0.1994711;  // = 1.0/sqrt(2*PI*sigma^2)

void main()
{
    vec4 result = vec4(0.0);
    vec2 texCoord;
    float gauss_func, div_factor = 0.0, k_shift;
    
    //Gauss filter
    int i,j;
	for(i = 0; i < kernel_size; i++)
	{
      k_shift = float(i-kernel_size/2);
#ifdef HORIZONTAL
      texCoord = fragTexCoord + vec2(k_shift*blur_radius, 0.0);
#else
      texCoord = fragTexCoord + vec2(0.0, k_shift*blur_radius);
#endif
      gauss_func = frac_sqrt_sigma*exp(-(abs(k_shift)*abs(k_shift))/sigma);
      div_factor += gauss_func;
      result +=  gauss_func * texture2D(bloomTexture, texCoord/2.0);
    }
    out_FragColor = result/div_factor;
}
