//GLSL vertex shader generated by gluxEngine
//Material: "mat_ground"

/*
//vertex attributes
in vec3 in_Vertex;
in vec3 in_Normal;
in vec2 in_Coord;
*/
 
//built-in vertex attributes
vec3 in_Vertex = gl_Vertex.xyz;
vec3 in_Normal = gl_Normal;
vec2 in_Coord = gl_MultiTexCoord0.st;

//material settings
struct Material{
  vec3 ambient;
  vec3 diffuse;
  vec3 specular;
  float shininess;
};
uniform Material material;

//texture coordinate
out vec2 fragTexCoord;
uniform float mat_groundBumpA_tileX, mat_groundBumpA_tileY, mat_groundBumpA_intensity;
uniform sampler2D mat_groundBumpA;
uniform float mat_groundDisplaceA_tileX, mat_groundDisplaceA_tileY, mat_groundDisplaceA_intensity;
uniform sampler2D mat_groundDisplaceA;
out vec4 v_color;
uniform int LIGHTS;

uniform float lightRadius;
uniform vec3 camPos;

//Calculate light model
vec4 LightModel(in vec3 normal, in vec3 eyeVec)
{
  vec3 final_color = vec3(0.0);
  vec3 N = normalize(normal);
  vec3 E = normalize(eyeVec);
  vec3 lightDir, L;
  float lambertTerm, specular, distSqr, att;
  						
  for(int i=0; i<LIGHTS; i++)
  {
    final_color += gl_LightSource[i].ambient.rgb * material.ambient;
    lightDir = (gl_LightSource[i].position.xyz + eyeVec + camPos)/(lightRadius);
    att = max(0.0, 1.0 - dot(lightDir, lightDir));
    L = normalize(lightDir);

  	lambertTerm = dot(N,L);
  	if(lambertTerm > 0.0)
  	{
  		final_color += gl_LightSource[i].diffuse.rgb * material.diffuse * lambertTerm * att;
  		vec3 R = reflect(-L, N);
  		specular = pow( max(dot(R, E), 0.0), material.shininess );
  		final_color += gl_LightSource[i].specular.rgb *  material.specular * specular * att;
  	}
  }
	
  return vec4(final_color,0.0);
} 


void main()
{
  vec4 vertex = vec4(in_Vertex,1.0);
  vec3 dNormal = in_Normal;
  
  //displace normal using normal map
  dNormal += normalize(texture2D(mat_groundBumpA,in_Coord*vec2(mat_groundBumpA_tileX, mat_groundBumpA_tileY)).rgb * 2.0 - 1.0);
  //displace vertex
  vec4 dv = texture2DLod(mat_groundDisplaceA, in_Coord*vec2(mat_groundDisplaceA_tileX, mat_groundDisplaceA_tileY), 0.0);
  float displace = dv.r; //dot( vec4(0.30,0.59,0.11,0.0),dv);
  vertex.xyz += mat_groundDisplaceA_intensity * displace * in_Normal;

  vec3 normal = gl_NormalMatrix * dNormal;            //surface normal
  vec3 vVertex = vec3(gl_ModelViewMatrix * vertex);  //view vector
  vec3 eyeVec = -vVertex;                               //eyeview position
  v_color = LightModel(normal,eyeVec);         //calculate light model
  fragTexCoord = in_Coord;
  gl_Position = gl_ModelViewProjectionMatrix * vertex;      //vertex transform
}

