/*
//vertex attributes
in vec3 in_Vertex;
in vec3 in_Normal;
in vec2 in_Coord;
*/

//built-in vertex attributes
vec3 in_Vertex = gl_Vertex.xyz;
vec3 in_Normal = gl_Normal;
vec2 in_Coord = gl_MultiTexCoord0.st;

//material settings
struct Material{
  vec3 ambient;
  vec3 diffuse;
  vec3 specular;
  float shininess;
};
uniform Material material;
		
//varying vec3 normal;
out vec4 vcolor;
out vec2 geomTexCoord;

void main()
{
    vec4 vertex = vec4(in_Vertex,0.5);
    vec3 normal = vec3(gl_ModelViewMatrix * vec4(in_Normal,0.0));
    geomTexCoord = in_Coord;
    gl_Position = vertex;
    
    //light model
    vec3 eyeVec = -vec3(gl_ModelViewMatrix * vertex);
    vcolor.rgb = gl_LightSource[0].ambient.rgb * material.ambient;
    vec3 N = normalize(normal);
    vec3 L = normalize(gl_LightSource[0].position.xyz);
    vec3 E = normalize(eyeVec);
    vcolor.rgb += gl_LightSource[0].diffuse.rgb * material.diffuse * dot(N,L);
    vec3 R = reflect(-L, N);
    float specular = pow( max(dot(R, E), 0.0), material.shininess );
    vcolor.rgb += gl_LightSource[0].specular.rgb *  material.specular * specular;
}
