//Inspired by:
//http://lumina.sourceforge.net/Tutorials/Noise.html
//http://freespace.virgin.net/hugo.elias/models/m_perlin.htm
in vec2 fragTexCoord;
out vec4 out_FragColor;
uniform float clamp_top, clamp_bottom, octaves, anim;

#define STEP 0.0035

//Returns random 4-dimensional vector
vec4 rand(vec2 A,vec2 B,vec2 C,vec2 D)
{
    vec2 s = vec2 (12.9898,78.233);
    vec4 tmp = vec4( dot(A,s),dot(B,s),dot(C,s),dot(D,s));
 
	return fract(sin(tmp)  * 43758.5453);
 }

//Calculate basic linear-filtered noise
float noise(vec2 coord,float d)
{
    vec2 C[4];
    float d1 = 1.0/d;
    C[0] = floor(coord * d)*d1 ;
    C[1] = C[0] + vec2(d1 ,0.0 );
    C[2] = C[0] + vec2(d1 ,d1);
    C[3] = C[0] + vec2(0.0,d1);

    vec2 p = fract(coord * d);
    vec2 q = 1.0 - p;
    vec4 w = vec4(q.x * q.y, p.x * q.y, p.x * p.y, q.x * p.y);

    return dot(rand(C[0],C[1],C[2],C[3]),w);
 }

//Creates 2D Perlin noise 
float perlinNoise2D(vec2 coord, float persistence, float oct)
{
	float total = 0.0;
    
    for(float i=1.0; i<=oct; i++)
    {
        float frequency = pow(2.0,i);
        float amplitude = pow(persistence,i);

        total += noise(coord*frequency+anim, amplitude)/oct;
    }
    return total;
}

void main(void)
{
	//1st, smooth cloud
	float cloud1 = perlinNoise2D(fragTexCoord,2.0,octaves)/2.0
				 + perlinNoise2D(fragTexCoord - STEP,2.0,octaves)/4.0;
				 + perlinNoise2D(fragTexCoord + STEP,2.0,octaves)/4.0;
	
	//2nd, background cloud
	float cloud2 = perlinNoise2D(fragTexCoord,2.0,octaves/2.0);
	//final clouds mixed from both
	float final_cloud = mix(cloud1, cloud2, 0.5);
	//sky color
	vec4 haze = vec4(1.0-fragTexCoord.y/2.0,1.0 - fragTexCoord.y/3.0,1.0,0.0);
	//final composition
	vec4 skycolor = haze + smoothstep(clamp_bottom,clamp_top,final_cloud);
	
	if(skycolor.r > 0.99)
		out_FragColor = mix(skycolor,vec4(cloud1),0.3);
	else
		out_FragColor = skycolor;
} 
