
in vec3 normal, lightDir, eyeVec;
in vec2 fragTexCoord;
out vec4 out_FragColor;

const vec3  SurfaceColor = vec3(0.0,1.0,0.0);
const vec3  DiffuseColor = vec3(0.0,0.5,0.0);

const vec2 Scale = vec2(10.0,10.0);
const vec2 Threshold = vec2(0.2,0.2);

//material settings
struct Material{
  vec3 ambient;
  vec3 diffuse;
  vec3 specular;
  float shininess;
};
uniform Material material;

//Calculates light model
vec4 LightModel(in vec3 normal, in vec3 lightDir, in vec3 eyeVec)
{
 	vec3 N = normalize(normal);
 	vec3 E = normalize(eyeVec);
  	vec3 L = normalize(lightDir);

 	float lambertTerm, specular;

    vec3 final_color = vec3(0.0);

  	lambertTerm = dot(N,L);
  	if(lambertTerm > 0.0)
  	{
  		final_color += gl_LightSource[0].diffuse.rgb * material.diffuse * lambertTerm;
  		vec3 R = reflect(-L, N);
  		specular = pow( max(dot(R, E), 0.0), material.shininess );
  		final_color += gl_LightSource[0].specular.rgb *  material.specular * specular;
  	}

  	return vec4(final_color,0.0);
}

void main(void)
{
   float ss = fract(fragTexCoord.s * Scale.s);
   float tt = fract(fragTexCoord.t * Scale.t);
   vec3 finalColor;

   if ((ss > Threshold.s) && (tt > Threshold.t)) 
   	discard;
   else
   	finalColor = SurfaceColor * DiffuseColor * LightModel(normal, lightDir, eyeVec).xyz;

   out_FragColor = vec4 (finalColor, 1.0);

}
