
in vec3 position, normal, lightDir, eyeVec;
out vec4 out_FragColor;

//material settings
struct Material{
  vec3 ambient;
  vec3 diffuse;
  vec3 specular;
  float shininess;
};
uniform Material material;

//PERLIN NOISE FUNCTIONS
//Inspired by:
//http://lumina.sourceforge.net/Tutorials/Noise.html
//http://freespace.virgin.net/hugo.elias/models/m_perlin.htm

//Returns pseudorandom 4-dimensional vector
vec4 rand(in vec3 A,in vec3 B,in vec3 C,in vec3 D)
{
    vec3 s = vec3 (12.9898,78.233, 37.13);
    vec4 tmp = vec4( dot(A,s),dot(B,s),dot(C,s),dot(D,s));
 
	return fract(sin(tmp) * 43758.5453);
 }
 
 
//Calculate basic linear-filtered noise
float noise(in vec3 coord,in float d)
{
    vec3 C[8];
    float d1 = 1.0/d;
    C[0] = floor(coord * d)*d1 ;	//actual sample
    //surrounding samples
    C[1] = C[0] + vec3(d1 ,0.0, 0.0 );
    C[2] = C[0] + vec3(d1 , d1, 0.0);
    C[3] = C[0] + vec3(0.0, d1, 0.0);
    C[4] = C[0] + vec3(0.0, 0.0, d1);
    C[5] = C[0] + vec3(d1 , 0.0, d1 );
    C[6] = C[0] + vec3(d1 ,  d1, d1);
    C[7] = C[0] + vec3(0.0,  d1, d1);
    
    //linear filtering
    vec3 p = fract(coord * d);
    vec3 q = 1.0 - p;
    vec4 w = vec4(q.x * q.y, p.x * q.y, p.x * p.y, q.x * p.y);

    return fract(dot(rand(C[0],C[1],C[2],C[3]),w)*dot(rand(C[4],C[5],C[6],C[7]),w));
 }

//Creates 3D Perlin noise with octaves
float perlinNoise3D(in vec3 coord, in float persistence, in float octaves)
{
	float total = 0.0;
    
    for(float i=1.0; i<=octaves; i++)
    {
        float frequency = pow(2.0,i);
        float amplitude = pow(persistence,i);

        total += noise(coord*frequency, amplitude)/octaves;
    }
    return total;
}


//Calculates light model
vec4 LightModel(in vec3 normal, in vec3 lightDir, in vec3 eyeVec)
{
 	vec3 N = normalize(normal);
 	vec3 E = normalize(eyeVec);
  	vec3 L = normalize(lightDir);

 	float lambertTerm, specular;

    vec3 final_color = vec3(0.0);

  	lambertTerm = dot(N,L);
  	if(lambertTerm > 0.0)
  	{
  		final_color += gl_LightSource[0].diffuse.rgb * material.diffuse * lambertTerm;
  		vec3 R = reflect(-L, N);
  		specular = pow( max(dot(R, E), 0.0), material.shininess );
  		final_color += gl_LightSource[0].specular.rgb *  material.specular * specular;
  	}

  	return vec4(final_color,0.0);
}


void main(void)
{
	float intensity = perlinNoise3D(position, 2.0, 5.0);
	
	vec3 color1 = vec3(0.6,0.6,1.0);
	vec3 color2 = vec3(0.75,0.75,1.0);

    float sineval = sin(length(position) * 10.0 + intensity * 10.0);
    	
	out_FragColor = vec4(mix(color1, color2, sineval),1.0)*LightModel(normal,lightDir,eyeVec);
} 
