uniform samplerCube mat_skyCubeA;
in vec3 normal, eyeVec;
out vec4 out_FragColor;

const float reflStr = 1.0;
const float refrStr = 1.0;
const float refrIndex = 1.4;

#ifndef USE_BUILTIN
//"Manual" implementation of refract() from ATI Render Monkey examples
vec3 myRefract(in vec3 eye, in vec3 norm, in float index)
{
    // We'll use Snell's refraction law:
   // n  * sin(theta ) = n  * sin(theta )
   //  i            i     r            r

   // sin(theta )
   //          i
   float cosine = dot(eye, norm);
   float sine = sqrt(1.0 - cosine * cosine);

   // sin(theta )
   //          r
   float sine2 = clamp(index * sine, 0.0, 1.0);
   float cosine2 = sqrt(1.0 - sine2 * sine2);

   // Out of the sine and cosine of the angle between the
   // refraction vector and the normal we can construct the
   // refraction vector itself given two base vectors.
   // These two base vectors are the negative normal and
   // a tangent vector along the path of the incoming vector
   // relative to the surface.
   vec3 y = normalize(cross(cross(eye, norm), norm));

   // Final refraction vector
   return -(norm * cosine2 + y * sine2);
}
#endif

void main(void)
{
    vec3 fnormal = normalize(normal);
    vec3 feyeVec = normalize(eyeVec);
    
    //reflection
    vec3 reflVec = reflect(feyeVec, fnormal);
    vec4 reflection = textureCube(mat_skyCubeA, reflVec)*reflStr;

#ifdef CHROMA
    //chromatic aberation
    #ifdef USE_BUILTIN
        vec3 refrVec_r = refract(feyeVec, fnormal, refrIndex);
    #else
        vec3 refrVec_r = myRefract(feyeVec, fnormal, refrIndex);
    #endif
    float refraction_r = textureCube(mat_skyCubeA, refrVec_r).r * refrStr;

    #ifdef USE_BUILTIN
        vec3 refrVec_g = refract(feyeVec, fnormal, refrIndex + 0.05);
    #else
        vec3 refrVec_g = myRefract(feyeVec, fnormal, refrIndex + 0.05);
    #endif
    float refraction_g = textureCube(mat_skyCubeA, refrVec_g).g * refrStr;

    #ifdef USE_BUILTIN
        vec3 refrVec_b = refract(feyeVec, fnormal, refrIndex + 0.1);
    #else
        vec3 refrVec_b = myRefract(feyeVec, fnormal, refrIndex + 0.1);
    #endif
    float refraction_b = textureCube(mat_skyCubeA, refrVec_b).b * refrStr;

    vec4 refraction = vec4(refraction_r, refraction_g, refraction_b, 1.0);
#else
    //refraction
    #ifdef USE_BUILTIN
        vec3 refrVec = refract(feyeVec, fnormal, refrIndex);
    #else
        vec3 refrVec = myRefract(feyeVec, fnormal, refrIndex);
    #endif
    vec4 refraction = textureCube(mat_skyCubeA, refrVec)*refrStr;
#endif

    //Fresnel equation: reflection is stronger at edge, refraction from direct view
    //so calculate angle between normal and view vector
    float dist = dot(fnormal,feyeVec);
    float fresnel = sqrt(1.0 - dist*dist );
    
	out_FragColor = mix(refraction, reflection, fresnel);
}
