// Author: Juraj Vanek
// This shader performs final composition of HDR image with tonemapping and SSAO

out vec4 out_FragColor;
in vec2 fragTexCoord;
uniform sampler2D blurTexture, renderTexture;


// Control exposure with this value
uniform float exposure;
// How much bloom to add
uniform float bloomFactor;
// Max bright
uniform float brightMax;

void main()
{
    vec3 color = texture2D(renderTexture, fragTexCoord).rgb;
    vec3 colorBloom = texture2D(blurTexture, fragTexCoord/2.0).rgb;
    float ssao_term =  texture2D(blurTexture, fragTexCoord/2.0).a;

    // Add bloom to the image
    color += colorBloom * bloomFactor;
    
    // Perform simple tone-mapping
    //float Y = dot(vec3(0.30, 0.59, 0.11), color);
    float YD = exposure * (exposure/brightMax + 1.0) / (exposure + 1.0);
    color *= YD;

    color *= vec3(ssao_term);

    //out_FragColor = vec4(ssao_term);
    out_FragColor.rgb = color;
}
