
//Material: "mat_wet"
uniform sampler2D mat_wetBaseA;
uniform sampler2D mat_wetBumpA;
uniform sampler2D mat_wetBumpB;
uniform float time;

in vec3 normal, eyeVec;
in vec2 fragTexCoord;
out vec4 out_FragColor;

//material settings
struct Material{
  vec3 ambient;
  vec3 diffuse;
  vec3 specular;
  float shininess;
};
uniform Material material;

vec4 LightModel(in vec3 normal, in vec3 eyeVec)
{
	vec3 final_color = material.ambient;
 	vec3 N = normalize(normal); 
 	vec3 E = normalize(eyeVec);
  						
    final_color += gl_LightSource[0].ambient.rgb * material.ambient;
    vec3 L = normalize(gl_LightSource[0].position.xyz + eyeVec);

	final_color += gl_LightSource[0].diffuse.rgb * material.diffuse * dot(N,L);
	vec3 R = reflect(-L, N);
	float specular = pow( max(dot(R, E), 0.0), material.shininess );
	final_color += gl_LightSource[0].specular.rgb *  material.specular * specular;
	
	return vec4(final_color,0.0);
} 

uniform float rain_speed;
//const float rain_speed = 20.0;

void main()
{
  vec2 texCoord = 5.0*fragTexCoord.st;

  //pseudo-random generator
  float rnd = float(( 73*int(time*rain_speed) + 1) % 8051)*0.1;
      
  //bump-mapping
  vec3 bump_normal = normal + normalize(texture2D(mat_wetBumpA, 2.0*texCoord).xyz*2.0 - 1.0)
                            + 2.5*normalize(texture2D(mat_wetBumpB, 1.5*texCoord + rnd ).xyz*2.0 - 1.0);
  vec4 color = LightModel(bump_normal,eyeVec);
  color *= texture2D(mat_wetBaseA, 2.0*texCoord);

  //FINAL fragment color
  out_FragColor = color;
}

