/**
****************************************************************************************************
****************************************************************************************************
@file: camera.h
@brief initialization and manipulating with camera - declarations
****************************************************************************************************
***************************************************************************************************/
#ifndef _CAMERA_H_
#define _CAMERA_H_

#include "globals.h"
#include "mat.h"

/**
@class TCamera 
@brief scene camera class. Sets default scene camera, performs basic camera movements, pointing to object etc...
***************************************************************************************************/
class TCamera
{
    protected:
				///camera position
        TVector pos;
				///camera rotation
				TVector rot;
				///camera look
				TVector look;
				///camera projection matrix
				TMatrix projection;
				///camera modelview matrix
				TMatrix modelview;
				///camera transform matrix
				TMatrix transform;

    public:
        TCamera();
        ~TCamera(){}
        //move camera
        TMatrix Move(GLfloat wx, GLfloat wy, GLfloat wz);
        TMatrix MoveAbs(GLfloat wx, GLfloat wy, GLfloat wz);
        //rotate camera
        TMatrix Rotate(GLfloat angle, GLint axis);
        TMatrix RotateAbs(GLfloat angle, GLint axis);
        //camera look
        TMatrix LookAt(GLfloat wx, GLfloat wy, GLfloat wz);
				//update camera matrix
				void UpdateMatrix();

        ///@brief get camera position
        TVector GetPos() { return pos; }
				///@brief get camera rotation
        TVector GetRot() { return rot; }
				///@brief get camera look
        TVector GetLook() { return look; }
        
        ///@brief Reset camera position
        void Reset() 
        { pos.Set(0,0,0,0); rot.Set(0,0,0,0); look.Set(0,0,0,0); }

        ///@brief Render camera: absolute or by looking position
        void RenderCamera() 
			  { glMultMatrixf(transform); }
};

#endif
