/**
****************************************************************************************************
****************************************************************************************************
@file globals.h
@brief common OpenGL and system includes, extern variables and enum types
****************************************************************************************************
***************************************************************************************************/
#ifndef _GLOBALS_H_
#define _GLOBALS_H_

//OpenGL version
//#define GL3

//Should we use OpenCL?
#define USE_CL


///Windows libraries
#if defined (__WIN32__) || defined(_WIN32) || defined(__CYGWIN__)
  #define _WIN_
  #include <windows.h>
#else
///Linux libraries
	#include <sys/time.h>
#endif

//SDL
#ifdef _WIN_
  #include "../extern/SDL/SDL.h"  //use library included here
#else
  #include <SDL/SDL.h>
#endif

///OpenGL/GLEW libraries
#include "../extern/glew/glew.h"
#include <GL/glu.h>

///OpenCL library
#ifdef USE_CL
	#include "../extern/CL/cl.h"
	#include "../extern/CL/cl_gl.h"
#endif

///C/C++ system libraries
#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <malloc.h>
#include <memory.h>
#include <string>
#include <fstream>
#include <iostream>
#include <sstream>
#include <vector>
#include <map>
#include <cmath>

using namespace std;
///Error code
const int ERR = -1;					

///Axis of rotation/scale
enum Axes {A_X, A_Y, A_Z};  

///Light types
enum LightModels{GOURAUD,PHONG,NONE};
///Light components
enum LightComponents{AMBIENT,DIFFUSE,SPECULAR};

///Texture types
enum TextureTypes{BASE,ENV,BUMP,PARALLAX,DISPLACE,CUBEMAP,CUBEMAP_ENV, ALPHA,
                  SHADOW,RENDER_TEXTURE,NORMAL_TEXTURE,BLOOM_TEXTURE, BLUR_TEXTURE};
///Texture mixing mods
enum TextureMods{ADD,MODULATE,DECAL,BLEND,REPLACE};

///Font types
enum font_type{NORMAL,TIMES,HELVETICA};
///Font sizes
enum font_size{SMALL,MEDIUM,LARGE};

/**
****************************************************************************************************
@brief Show message box (or output to console on Linux)
@param msg message to be shown
@param info if true, show information box. Else show error box
****************************************************************************************************/
inline void ShowMessage(const char *msg,bool info=true)
{
  if(!info)
  {
  #ifdef _WIN_
    MessageBox(NULL,msg,"ERROR",MB_ICONERROR);
  #else
    cerr<<"ERROR: "<<msg<<endl;
  #endif
  }
  else
  {
  #ifdef _WIN_
    MessageBox(NULL,msg,"Info",MB_ICONINFORMATION);
  #else
    cout<<msg<<endl;
  #endif
  }
}

/**
****************************************************************************************************
@brief Convert number into C++ string
@param t number to be converted
@return string with number
****************************************************************************************************/
template <class T> inline string num2str(const T& t)
{
	stringstream ss;
	ss << t;
	return ss.str();
}


#endif
