///-----------------------------------------------------------------
///
/// @file      change_dialog.cpp
/// @author    Juraj Vanek
/// Created:   16. 4. 2010 0:31:39
/// @section   DESCRIPTION
///            ChangeDialog class implementation
///
///------------------------------------------------------------------

#include "change_dialog.h"

//Do not add custom headers
//wxDev-C++ designer will remove them
////Header Include Start
////Header Include End

//----------------------------------------------------------------------------
// ChangeDialog
//----------------------------------------------------------------------------
//Add Custom Events only in the appropriate block.
//Code added in other places will be removed by wxDev-C++
////Event Table Start
BEGIN_EVENT_TABLE(ChangeDialog,wxDialog)
	////Manual Code Start
	////Manual Code End
	
	EVT_CLOSE(ChangeDialog::OnClose)
	EVT_BUTTON(ID_BTNCHANGEOK,ChangeDialog::btnChangeOKClick)
END_EVENT_TABLE()
////Event Table End

ChangeDialog::ChangeDialog(TForm *parent, wxWindowID id, const wxString &title, const wxPoint &position, const wxSize& size, long style)
: wxDialog(parent, id, title, position, size, style)
{
	CreateGUIControls();
	p_parent = parent;
	
	//find out all supported resolutions
  wxDisplay display;
  wxArrayVideoModes modes = display.GetModes();
  
  wxString tmp;

  //if autodetection fails, add resolution manually
  if(modes.GetCount() <= 1)
  {
    choiceRes->Append(wxT("1024x768"));
    choiceRes->Append(wxT("1280x720"));
    choiceRes->Append(wxT("1680x1050"));
    choiceRes->Append(wxT("1920x1080"));
  }
  else  //autodetection
  {
    
    for(unsigned i=0; i<modes.GetCount(); i++)
    {
      //Linux bug: returns video modes, but with zero resolution
      if(modes[i].w == 0 && modes[i].h == 0)
      {
      	choiceRes->Append(wxT("1024x768"));
      	choiceRes->Append(wxT("1280x720"));
      	choiceRes->Append(wxT("1680x1050"));
      	choiceRes->Append(wxT("1920x1080"));	
      	break;
      }
      //select only resolution bigger than 800x60, 32bit, 60Hz
      if(modes[i].w >= 800 && modes[i].bpp > 16 && modes[i].w > modes[i].h && modes[i].refresh == 60)
      {
        //add to choice control
        wxString tmp;
        tmp<<modes[i].w<<wxT("x")<<modes[i].h;
        choiceRes->Append(tmp);
      }
    }
  }
  choiceRes->SetSelection(0);
}

ChangeDialog::~ChangeDialog()
{
} 

void ChangeDialog::CreateGUIControls()
{
	//Do not add custom code between
	//GUI Items Creation Start and GUI Items Creation End.
	//wxDev-C++ designer will remove them.
	//Add the custom code before or after the blocks
	////GUI Items Creation Start

	checkWindowed = new wxCheckBox(this, ID_CHECKWINDOWED, wxT("Run in window"), wxPoint(88, 32), wxSize(97, 17), 0, wxDefaultValidator, wxT("checkWindowed"));

	btnChangeOK = new wxButton(this, ID_BTNCHANGEOK, wxT("OK"), wxPoint(80, 138), wxSize(75, 25), 0, wxDefaultValidator, wxT("btnChangeOK"));

	wxArrayString arrayStringFor_choiceSeries;
	arrayStringFor_choiceSeries.Add(wxT("All tests"));
	arrayStringFor_choiceSeries.Add(wxT("Fillrate"));
	arrayStringFor_choiceSeries.Add(wxT("Fragment shaders"));
	arrayStringFor_choiceSeries.Add(wxT("Geometry shaders"));
	arrayStringFor_choiceSeries.Add(wxT("Vertex shaders"));
	arrayStringFor_choiceSeries.Add(wxT("Complex tests"));
	arrayStringFor_choiceSeries.Add(wxT("Compute tests"));
	choiceSeries = new wxChoice(this, ID_CHOICESERIES, wxPoint(88, 96), wxSize(120, 23), arrayStringFor_choiceSeries, 0, wxDefaultValidator, wxT("choiceSeries"));
	choiceSeries->SetSelection(0);

	WxStaticText4 = new wxStaticText(this, ID_WXSTATICTEXT4, wxT("Test series:"), wxPoint(8, 96), wxDefaultSize, 0, wxT("WxStaticText4"));

	wxArrayString arrayStringFor_choiceAA;
	arrayStringFor_choiceAA.Add(wxT("Off"));
	arrayStringFor_choiceAA.Add(wxT("2x"));
	arrayStringFor_choiceAA.Add(wxT("4x"));
	arrayStringFor_choiceAA.Add(wxT("8x"));
	choiceAA = new wxChoice(this, ID_CHOICEAA, wxPoint(88, 64), wxSize(120, 23), arrayStringFor_choiceAA, 0, wxDefaultValidator, wxT("choiceAA"));
	choiceAA->SetSelection(0);

	WxStaticText3 = new wxStaticText(this, ID_WXSTATICTEXT3, wxT("AntiAliasing:"), wxPoint(8, 64), wxDefaultSize, 0, wxT("WxStaticText3"));

	wxArrayString arrayStringFor_choiceRes;
	choiceRes = new wxChoice(this, ID_CHOICERES, wxPoint(88, 8), wxSize(120, 23), arrayStringFor_choiceRes, 0, wxDefaultValidator, wxT("choiceRes"));
	choiceRes->SetSelection(-1);

	WxStaticText2 = new wxStaticText(this, ID_WXSTATICTEXT2, wxT("Resolution:"), wxPoint(8, 8), wxDefaultSize, 0, wxT("WxStaticText2"));

	SetTitle(wxT("Change settings"));
	SetIcon(wxNullIcon);
	SetSize(8,8,240,214);
	Center();
	
	////GUI Items Creation End
}

void ChangeDialog::OnClose(wxCloseEvent& /*event*/)
{
	Destroy();
}

/*
 * Close window
 */
void ChangeDialog::btnChangeOKClick(wxCommandEvent& event)
{
  //update settings in main window
  wxString curr_res = choiceRes->GetStringSelection();
  int index = curr_res.find(wxChar('x'));
  p_parent->UpdateSettings(curr_res.SubString(0,index - 1), curr_res.SubString(index+1, curr_res.Length() - 1),
                           choiceAA->GetStringSelection(), choiceSeries->GetStringSelection(), checkWindowed->IsChecked());
	EndModal(0);
}
