/******************************************************************************/
//
//  Program: gluxMark2 2009, DIP
//  Author: Juraj Vanek, xvanek29@stud.fit.vutbr.cz
//  File: test_complex.cpp
//  Desc: implementation of complex test series
//
//*****************************************************************************/
#include "test_complex.h"

/**
****************************************************************************************************
@brief Complex tests initialization
@param _duration test duration
@param _type test type
@param s pointer to scene
****************************************************************************************************/
TestComplex::TestComplex(int _duration, int _type, TScene *s) : Test(_duration, _type, s)
{
  //switch according to test subtype
  if(type == COMPLEX_HDR)
  {
	  testname = "complex_hdr";
		weight = 0.30f;
		splits = 2;									//HDR on/off
		s->SetLoadList(167);	//loaded items count (must be determined manually :(

		//keyframe animation (linear interpolation)
		key1_pos.Set(3.1f, -8.7f, -12.8f);
		key1_rot.Set(-25.0f, -180.0f, 0.0f);
    key2_pos.Set(2.2f, -2.7f, -22.2f);
		key2_rot.Set( 12.0f, -35.0f, 0.0f);
		inv = 1;
		fps = 100;

		try
		{
			//toggle HDR
			s->UseHDR(true);

			//prepare render to texture
			s->CreateRenderTarget();

			//position camera
			s->MoveCamera(key1_pos.x, key1_pos.y, key1_pos.z);
			s->RotateCamera(key1_rot.x, A_X);
			s->RotateCamera(key1_rot.y, A_Y);

			//lights
			s->AddLight(0, silver, white, white, TVector(30.0, 40.0, 30.0),1000.0);

			//cubemap files
			const char *cubemap[] = { "data/tex/cubemaps/posx.tga", "data/tex/cubemaps/negx.tga",
																"data/tex/cubemaps/posy.tga", "data/tex/cubemaps/negy.tga",
																"data/tex/cubemaps/posz.tga", "data/tex/cubemaps/negz.tga" };

			//load scenes
			s->LoadScene("data/obj/scenes/bmw.3ds");
			s->LoadScene("data/obj/scenes/hills.3ds");
			//additional materials/textures
			s->AddTexture("hmatroad","data/tex/road.tga",BASE, MODULATE, 1.0, 1.0, 1.0, true, true);
			s->AddTexture("hmatgrass","data/tex/grass.tga",BASE, MODULATE, 0.5, 10.0, 20.0, true, true);
			s->AddTexture("hmathills","data/tex/dirtgras.tga",BASE, MODULATE, 0.5, 10.0, 20.0);
			s->AddMaterial("mat_body",silver);
			s->AddTexture("mat_body",cubemap,CUBEMAP_ENV);
			s->SetMaterial("body","mat_body");
			s->AddMaterial("mat_glass",black, white, white, 64.0, 0.0, 1.0);
			s->AddTexture("mat_glass",cubemap,CUBEMAP_ENV);
			s->SetMaterial("glass","mat_glass");
			s->AddMaterial("mat_chrome",black, white, white);
			s->AddTexture("mat_chrome",cubemap,CUBEMAP_ENV,ADD);
			s->SetMaterial("metal","mat_chrome");

			//load objects
			s->AddObject("skybox",SPHERE,400.0,0.0,8,8);
			s->MoveObj("skybox",0.0, 100.0, 0.0);

			//add materials
			s->AddMaterial("mat_sky",white,white,white,0.0,0.0,0.0,NONE);
			s->AddMaterial("mat_bark",lgrey,white);
			s->AddMaterial("mat_leaf",lgrey,white);
			s->AddMaterial("mat_grass",green,white);

			//add textures
			s->AddTexture("mat_bark","data/tex/bark4.tga");
			s->AddTexture("mat_bark","data/tex/normal/bark4_nm.tga",BUMP,MODULATE, 1.0);
			s->AddTexture("mat_leaf","data/tex/alpha/leafs4.tga",ALPHA);
			s->AddTexture("mat_sky",cubemap);
			s->AddTexture("mat_grass","data/tex/alpha/grass.tga",ALPHA);

			//set materials
			s->SetMaterial("skybox","mat_sky");

			//trees&grass
			float rnd;
			int OBJECT_COUNT_X = 7;
			int OBJECT_COUNT_Y = 4;
			string bark, leaf;
			for(int i=0; i<OBJECT_COUNT_X; i++)
			{
				for(int j=0; j<OBJECT_COUNT_Y; j++)
				{
					rnd = rand()%100 / 25.0f - 2.0f;
					bark = "tree" + num2str(i*OBJECT_COUNT_X + j) + "_bark";
					leaf = "tree" + num2str(i*OBJECT_COUNT_Y + j) + "_leaf";
					//two tree types
					if( (i*OBJECT_COUNT_X + j)%2 == 1)
					{
						s->AddObject(bark.c_str(),"data/obj/trees/tree1_bark.3ds");
						s->AddObject(leaf.c_str(),"data/obj/trees/tree1_leaf.3ds");
					}
					else
					{
						s->AddObject(bark.c_str(),"data/obj/trees/tree2_bark.3ds");
						s->AddObject(leaf.c_str(),"data/obj/trees/tree2_leaf.3ds");
					}
					float movz = i*14.0f + rnd - 7.0f*OBJECT_COUNT_X + 10.0f;
					float movx = j*14.0f - rnd - 7.0f*OBJECT_COUNT_Y - 30.0f;
					float roty = 10.0f * rnd;
					s->MoveObj(bark.c_str(), movx, 2.0f-j, movz);
					s->MoveObj(leaf.c_str(), movx, 2.0f-j, movz);
					s->RotateObj(bark.c_str(), roty, A_Y);
					s->RotateObj(leaf.c_str(), roty, A_Y);
					s->SetMaterial(bark.c_str(), "mat_bark");
					s->SetMaterial(leaf.c_str(), "mat_leaf");
				}
			}

			//General HDR settings
			s->SetUniform("mat_bloom_hdr","THRESHOLD",0.3);
			s->SetUniform("mat_tonemap","exposure",1.0);
			s->SetUniform("mat_tonemap","bloomFactor",1.0);
			s->SetUniform("mat_tonemap","brightMax",1.5);

			//blur
			s->SetUniform("mat_blur_horiz","kernel_size",21);
			s->SetUniform("mat_blur_horiz","blur_radius",0.005);
			s->SetUniform("mat_blur_vert","kernel_size",21);
			s->SetUniform("mat_blur_vert","blur_radius",0.005);
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == COMPLEX_SSAO)
  {
    testname = "complex_ssao";
		weight = 0.30f;
		splits = 4;									//SSAO on/off
		//s->SetLoadList(1);	//loaded items count (must be determined manually :(

		try
		{
			//toggle SSAO
			s->UseSSAO(true);
			//prepare render to texture
			s->CreateRenderTarget();

			//load scene
			s->LoadScene("data/obj/scenes/room.3ds");

			//adjust some parameters
			s->MoveCameraAbs(6.5,-15.0,-90.0);
			s->RotateCamera(15.0,A_X);
			s->SetLightRadius(0,150.0);

			//General HDR settings
			s->SetUniform("mat_tonemap","exposure",1.2);
			s->SetUniform("mat_tonemap","bloomFactor",1.0);
			s->SetUniform("mat_tonemap","brightMax",1.0);

			//SSAO settings
			s->SetUniform("mat_bloom_ssao","totStrength",2.2);
			s->SetUniform("mat_bloom_ssao","rad",0.02);
			s->SetUniform("mat_bloom_ssao","bias",0.02);

			//blur
			s->SetUniform("mat_blur_horiz","kernel_size",15);
			s->SetUniform("mat_blur_horiz","blur_radius",0.002);
			s->SetUniform("mat_blur_vert","kernel_size",15);
			s->SetUniform("mat_blur_vert","blur_radius",0.002);
		}
		catch(int err)
		{
			stopbit = true;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == COMPLEX_SHADOW)
  {
    testname = "complex_shadow";
		weight = 0.30f;
		splits = 4;									//shadows on/off
		s->SetLoadList(8);	//loaded items count (must be determined manually :(

		//position camera
		s->MoveCamera(1.0,-3.0,-20.0);
		s->RotateCamera(5.0, A_X);
		s->RotateCamera(60.0, A_Y);

		try
		{
			lpos1.Set(-30.0,45.0,5.0);
			//lights
			s->AddLight(0,dgrey,dgrey,white,lpos1,1000.0);
			s->AddLight(1,dgrey,dgrey,grey,TVector(-20.0,20.0,10.0));
			s->AddLight(2,dgrey,dgrey,grey,TVector(-20.0,20.0,-10.0));
			s->SetShadow(0,4096);

			s->LoadScene("data/obj/scenes/chata.3ds");
			//additional materials&textures
			s->AddMaterial("mat_sky",white,white,white,1.0,0.0,0.0,NONE);
			s->AddTexture("shmatleaf","data/tex/alpha/leafs4.tga",ALPHA);
			s->AddTexture("shmatgrass","data/tex/alpha/grass.tga",ALPHA);
			s->AddTexture("shmatroad","data/tex/dirtroad.tga",BASE,MODULATE,1.0,1.0,1.0,true,true);
			s->AddTexture("shmatroad","data/tex/normal/bump.tga",BUMP,MODULATE,1.0,1.0,1.0,true,true);
			s->AddTexture("shmatroof","data/tex/roof.tga",BASE,MODULATE,1.0,1.0,1.0,true,true);
			const char *cubemap[] = { "data/tex/cubemaps/sky/posx.tga", "data/tex/cubemaps/sky/negx.tga",
																"data/tex/cubemaps/sky/posy.tga", "data/tex/cubemaps/sky/negy.tga",
																"data/tex/cubemaps/sky/posz.tga", "data/tex/cubemaps/sky/negz.tga" };
			s->AddTexture("mat_sky",cubemap);
			//load objects
			s->AddObject("sky",SPHERE,200.0,0.0,8,8);

			//set materials
			s->SetMaterial("sky","mat_sky");

			//cast/receive shadows
			s->CastShadow("sky",false);
			s->CastShadow("ground",false);
			s->ReceiveShadow("mat_sky",false);
		}
		catch(int err)
		{
			stopbit = true;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == COMPLEX_SSAMPLING)
  {
    testname = "complex_ssampling";
		weight = 0.10f;
		splits = 4;									//supersampling on/off
		s->SetLoadList(17 + 98 + 30);	//loaded items count (must be determined manually :(

		//position camera
		s->MoveCamera(0.0, -4.5, -15.0);
		s->RotateCamera(-7.0, A_X);

		try
		{
			//lights
			s->AddLight(0, lgrey, white, white, TVector(30.0, 40.0, 30.0));

			//toggle HDR
			s->UseHDR(true);

			//prepare render to texture
			s->CreateRenderTarget(2*s->GetResX(),2*s->GetResY(),GL_RGBA,GL_UNSIGNED_BYTE);

			//load scenes
			s->LoadScene("data/obj/scenes/forest.3ds");
			s->AddMaterial("mat_well",grey);
			s->AddTexture("mat_well","data/tex/wall.tga",BASE, MODULATE, 1.0, 4.0, 4.0);
			s->AddTexture("mat_well","data/tex/normal/wall.tga",BUMP, MODULATE, 1.0, 4.0, 4.0);
			s->SetMaterial("well","mat_well");

			//load objects
			s->AddObject("skybox",SPHERE,400.0,0.0,8,8);
			s->AddObject("ground",PLANE,200.0,200.0);

			//add materials
			s->AddMaterial("mat_bark",lgrey,white);
			s->AddMaterial("mat_leaf",lgrey,white);
			s->AddMaterial("mat_grass",lgrey,white);
			s->AddMaterial("mat_ground",silver,silver);
			s->AddMaterial("mat_forest",white, white);
			s->AddMaterial("mat_sky",white,white,white,0.0,0.0,0.0,NONE);

			//add textures
			s->AddTexture("mat_bark","data/tex/bark4.tga");
			s->AddTexture("mat_bark","data/tex/normal/bark4_nm.tga",BUMP,MODULATE, 1.0);
			s->AddTexture("mat_leaf","data/tex/alpha/leafs4.tga",ALPHA);
			s->AddTexture("mat_grass","data/tex/alpha/grass.tga",ALPHA);
			const char *cubemap[] = { "data/tex/cubemaps/sky/posx.tga", "data/tex/cubemaps/sky/negx.tga",
																"data/tex/cubemaps/sky/posy.tga", "data/tex/cubemaps/sky/negy.tga",
																"data/tex/cubemaps/sky/posz.tga", "data/tex/cubemaps/sky/negz.tga" };
			s->AddTexture("mat_sky",cubemap);
			s->AddTexture("mat_ground","data/tex/grass.tga",BASE, MODULATE, 1.0, 10.0, 20.0);
			s->AddTexture("mat_forest","data/tex/alpha/forest.tga",ALPHA,MODULATE, 1.0, 10.0);

			//set materials
			s->SetMaterial("skybox","mat_sky");
			s->SetMaterial("ground","mat_ground");
			s->SetMaterial("forest","mat_forest");

			//trees
			float rnd;
			int OBJECT_COUNT = 7;
			string bark, leaf;
			for(int i=0; i<OBJECT_COUNT; i++)
			{
				for(int j=0; j<OBJECT_COUNT; j++)
				{
					rnd = rand()%100 / 25.0f - 2.0f;
					bark = "tree" + num2str(i*OBJECT_COUNT + j) + "_bark";
					leaf = "tree" + num2str(i*OBJECT_COUNT + j) + "_leaf";
					//two tree types
					if( (i*OBJECT_COUNT + j)%2 == 1)
					{
						s->AddObject(bark.c_str(),"data/obj/trees/tree1_bark.3ds");
						s->AddObject(leaf.c_str(),"data/obj/trees/tree1_leaf.3ds");
					}
					else
					{
						s->AddObject(bark.c_str(),"data/obj/trees/tree2_bark.3ds");
						s->AddObject(leaf.c_str(),"data/obj/trees/tree2_leaf.3ds");
					}
					float movx = j*14.0f + rnd - 7.0f*OBJECT_COUNT;
					float movz = i*14.0f - rnd - 7.0f* OBJECT_COUNT;
					float roty = 10.0f * rnd;
					s->MoveObj(bark.c_str(), movx, 0.0, movz);
					s->MoveObj(leaf.c_str(), movx, 0.0, movz);
					s->RotateObj(bark.c_str(), roty, A_Z);
					s->RotateObj(leaf.c_str(), roty, A_Z);
					s->SetMaterial(bark.c_str(), "mat_bark");
					s->SetMaterial(leaf.c_str(), "mat_leaf");
				}
			}

			//grass - one object and then instances
			OBJECT_COUNT = 30;
			s->AddObject("grass","data/obj/trs.3ds");
			s->SetMaterial("grass", "mat_grass");
			string grass;
			for(int i=0; i<OBJECT_COUNT; i++)
			{
				for(int j=0; j<OBJECT_COUNT; j++)
				{
					rnd = rand()%100 / 25.0f - 2.0f;
					grass = "grass" + num2str(i*OBJECT_COUNT + j);
					s->AddObjectInstance("grass",grass.c_str());
					float movx = j*2.0f + rnd - OBJECT_COUNT;
					float movz = i*2.0f - rnd - OBJECT_COUNT;
					float roty = 10.0f * rnd;
					s->MoveObj(grass.c_str(), movx, 0.0, movz);
					s->RotateObj(grass.c_str(), roty, A_Z);
				}
			}

			//General HDR settings
			s->SetUniform("mat_bloom_hdr","THRESHOLD",0.0);
			s->SetUniform("mat_tonemap","exposure",0.5);
			s->SetUniform("mat_tonemap","bloomFactor",1.0);
			s->SetUniform("mat_tonemap","brightMax",1.0);

			//blur in supersample rendering
			s->SetUniform("mat_blur_horiz","kernel_size",1);
			s->SetUniform("mat_blur_horiz","blur_radius",0.0);
			s->SetUniform("mat_blur_vert","kernel_size",1);
			s->SetUniform("mat_blur_vert","blur_radius",0.0);
		}
		catch(int err)
		{
			stopbit = true;
			return;
		}
  }

  //done
  if(!s->PostInit())
		stopbit = true;
  InitTime();
}

//**************************************************************************************************
//Test deinitialization, reset scene, delete dynamic data, save results
TestComplex::~TestComplex()
{
  s->Destroy(true);
}

//**************************************************************************************************
//Test run
void TestComplex::Run()
{
  Test::Run();
	if(!draw_scene)
		return;   //don't draw scene if has ended or splashscreen is present

  /////////////////////////////////////////////////////////////////////////////////////////////////
  //switch according to test type
  if(type == COMPLEX_HDR)
  {
    if(splitbit)   //split bit set, save partial results and toggle HDR
    {
      splitbit = false;

      //set lights for actual partial scene
      switch(curr_setup)
      {
        //HDR off
        case 1:
					s->UseHDR(false);
					msg = "HDR lighting OFF";
					break;
				//SSAO on
				case 2:
					s->UseHDR(true);
					inv = -1;
					msg = "HDR lighting ON";
					break;
				default: break;
			}
		}
		s->MoveCamera(dp.x, dp.y, dp.z);
		s->RotateCamera(dr.x, A_X);
		s->RotateCamera(dr.y, A_Y);

		s->Redraw();

		//info screen
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == COMPLEX_SSAO)
  {
    if(splitbit)   //split bit set, save partial results and toggle SSAO
    {
      splitbit = false;

      //set lights for actual partial scene
      switch(curr_setup)
      {
        //SSAO off
        case 1:
					s->UseSSAO(false);
					msg = "Ambient occlusion OFF";
					break;
				//SSAO on
				case 2:
					try 
					{ 
						s->UseSSAO(true); 
					}
					catch(int err)
					{
						stopbit = true;
						return;
					}
					msg = "Ambient occlusion ON";
					break;
				default: break;
			}
		}
		//animation
		s->RotateCameraAbs(anim*10.0f,A_Y);
		s->MoveObjAbs("motor",0.0f,sin(5.0f*anim), 0.0f);
		s->MoveObjAbs("motor_fan",-32.5f, 14.0f + sin(5.0f*anim), -4.5f);
		s->RotateObjAbs("motor_fan",anim*100.0f,A_X);

		s->Redraw();
		//info screen
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == COMPLEX_SHADOW)
  {
    if(splitbit)   //split bit set, save partial results and toggle shadows
    {
      splitbit = false;

      //set lights for actual partial scene
      switch(curr_setup)
      {
        //shadows off
        case 1:
					s->UseShadows(false);
					msg = "Dynamic shadows OFF";
					break;
				//shadows on
				case 2:
					s->UseShadows(true);
					msg = "Dynamic shadows ON";
					break;
				default: break;
			}
		}
		if(curr_setup > 1)
		{
			//move light and change intensity
			float t_anim = (anim - 7.5f - SPLASH/1000.0f)/7.5f + PI/2.0f;
			lpos1.z = 50.0f * sin(t_anim);
			lpos1.y = 30.0f * abs(cos(t_anim)) + 10.0f;
			float l_color = abs(cos(t_anim)) + 0.1f;
			s->MoveLight(0,lpos1);
			s->ChangeLightColor(0,DIFFUSE,TVector(l_color,l_color,l_color));
			s->ChangeLightColor(1,DIFFUSE,TVector(l_color,l_color,l_color));
			s->ChangeLightColor(2,DIFFUSE,TVector(l_color,l_color,l_color));
		}
		//move camera
		s->MoveCamera(0.0,0.0,-0.15f/fps);

		s->Redraw();
		//info screen
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
	/////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == COMPLEX_SSAMPLING)
  {
    if(splitbit)   //split bit set, save partial results and toggle supersampling (through HDR render)
    {
      splitbit = false;

      //set lights for actual partial scene
      switch(curr_setup)
      {
        //supesampling off
        case 1:
					s->UseHDR(false);
					msg = "Supersampling OFF";
					break;
				//SSAO on
				case 2:
					s->UseHDR(true);
					msg = "Supersampling ON";
					break;
				default: break;
			}
		}
		//animation
		s->RotateCameraAbs(anim*5.0f, A_Y);

		s->Redraw();
		//info screen
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
}

//*************************************************************************************************
//Scene animation (every 10ms)
void TestComplex::Animate()
{
  Test::Animate();

	switch(type)
	{
	case COMPLEX_HDR:
		//keyframe diff
		dp = (key2_pos - key1_pos)/(float)fps/12.0f * (float)inv;
		dr = (key2_rot - key1_rot)/(float)fps/12.0f * (float)inv;
		break;
	default:
		break;
	}
}


//**************************************************************************************************
//Save results to XML file
void TestComplex::SaveResults()
{
  Test::SaveResults();

  if(type == COMPLEX_HDR)
	{
    //write performance for HDR on/off
    switch(curr_setup)
    {
      case 1:
          results<<"<complex>\n"<<
						     "<"<<testname<<">\n"<<
                 "<hdr_off>\n<fps>"<<avg_fps<<"</fps>\n</hdr_off>\n";
          break;
      case 2:
					//final weight
					test_score = avg_fps*4;
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<hdr_on>\n<fps>"<<avg_fps<<"</fps>\n</hdr_on>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
	}

  else if(type == COMPLEX_SSAO)
  {
    //write performance for SSAO on/off
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<ao_off>\n<fps>"<<avg_fps<<"</fps>\n</ao_off>\n";
          break;
      case 4:
					//final weight
					test_score = avg_fps * 4.5f;
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<ao_on>\n<fps>"<<avg_fps<<"</fps>\n</ao_on>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == COMPLEX_SHADOW)
  {
    //write performance for shadows on/off
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<shadow_off>\n<fps>"<<avg_fps<<"</fps>\n</shadow_off>\n";
          break;
      case 4:
					//final weight
					test_score = avg_fps * 10.0f;
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<shadow_on>\n<fps>"<<avg_fps<<"</fps>\n</shadow_on>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == COMPLEX_SSAMPLING)
  {
    //write performance for supersampling on/off
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<ssampling_off>\n<fps>"<<avg_fps<<"</fps>\n</ssampling_off>\n";
          break;
      case 4:
					//final weight
					test_score = avg_fps * 3.3f;
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<ssampling_on>\n<fps>"<<avg_fps<<"</fps>\n</ssampling_on>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n"
							"</complex>\n";
          break;
      default: break;
    }
  }
}
