/******************************************************************************/
//
//  Program: gluxMark2, DIP
//  Author: Juraj Vanek, xvanek29@stud.fit.vutbr.cz
//  File: test_vert.cpp
//  Desc: implementation of vertex shaders test series
//
//*****************************************************************************/
#include "test_vert.h"

/**
****************************************************************************************************
@brief Vertex shader tests initialization
@param _duration test duration
@param _type test type
@param s pointer to scene
****************************************************************************************************/
TestVert::TestVert(int _duration, int _type, TScene *s) : Test(_duration, _type, s)
{
  //switch according to test subtype
  if(type == VERT_LIGHTS)
  {
	  testname = "vert_lights";
		weight = 0.45f;
    splits = 3;
		s->SetLoadList(4);	//loaded items count (must be determined manually :(
		int INSTANCES = 20;

	  //camera position
		s->RotateCamera(-45.0,A_Y);
		s->RotateCamera(20.0,A_X);

	  //basic light positions
	  lpos1.Set(0.0,10.0,0.0); lpos2.Set(0.0,10.0,0.0); lpos3.Set(0.0,10.0,0.0); lpos4.Set(0.0,10.0,0.0);
	  lpos5.Set(0.0,10.0,0.0); lpos6.Set(0.0,10.0,0.0); lpos7.Set(0.0,10.0,0.0); lpos8.Set(0.0,10.0,0.0);

		//add 1 starting light
	  s->AddLight(0,black,white,white,lpos1,40.0);

		try
		{
			//car
			s->LoadScene("data/obj/scenes/bmw.3ds");
			s->DrawObject("glass",false);
			s->AddMaterial("mat_body",black,TVector(0.0f,0.0f,0.02f),white,64.0,0.0,0.0,GOURAUD);
			s->SetMaterial("body","mat_body");

			//ground
			s->AddMaterial("mat_tile",black,white,white,8.0,0.0,0.0,GOURAUD);
			s->AddTexture("mat_tile","data/tex/metal.tga");


			s->AddObject("tile0","data/obj/tile.3ds");
			s->MoveObj("tile0", -INSTANCES*2.0f, -1.7f, -INSTANCES*2.0f);
			s->SetMaterial("tile0", "mat_tile");
			//geometry instanced floor (if supported)
      if(GLEW_ARB_draw_instanced || GLEW_EXT_draw_instanced)
      {
			  s->CustomShader("mat_tile","data/shaders/instanced.vert",NULL,"data/shaders/instanced.frag");
			  s->SetUniform("mat_tile","INSTANCES",INSTANCES);
			  s->SetGInstances("tile0",INSTANCES*INSTANCES);
      }
      //else use less effective method of instancing
      else
      {
        string obj_name;
        for(int i=0; i<INSTANCES*INSTANCES; i++)
			  {
				  obj_name = "tile0" + num2str(i + 1);
				  s->AddObjectInstance("tile0",obj_name.c_str());
				  s->MoveObj(obj_name.c_str(), 5.0f*(i%INSTANCES), 0.0f, 5.0f*(i/INSTANCES));
			  }
      }
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
		total_verts = 56847 + INSTANCES * INSTANCES * 4912;		//total scene vertices
	}
	else if(type == VERT_WAVES)
  {
	  testname = "vert_waves";
		weight = 0.25;
    splits = 1;
		s->SetLoadList(7);	//loaded items count (must be determined manually :(

		 //camera and light position
		s->MoveCamera(0.0,-5.0,-5.0);

		try
		{
			//add and position objects
			s->AddObject("sky",SPHERE,200.0,0.0,8,8);
			s->AddObject("sea",STRIP_PLANE,500.0,500.0,500,500);

			//add lights
			s->AddLight(0,grey,white,white,TVector(0.5,20.0,1.5));

			//add materials & textures
			s->AddMaterial("mat_sea",silver,white,white,16.0);
			s->AddMaterial("mat_sky",black,white,white,0.0,0.0,0.0,NONE);
			const char *cubemaps[] = { "data/tex/cubemaps/sky/posx.tga", "data/tex/cubemaps/sky/negx.tga", "data/tex/cubemaps/sky/posy.tga",
																 "data/tex/cubemaps/sky/negy.tga", "data/tex/cubemaps/sky/posz.tga", "data/tex/cubemaps/sky/negz.tga" };
			s->AddTexture("mat_sky",cubemaps);
			s->AddTexture("mat_sea",cubemaps,CUBEMAP_ENV);
			s->AddTexture("mat_sea","data/tex/sea.tga");

			s->CustomShader("mat_sea","data/shaders/sea.vert",NULL,"data/shaders/sea.frag");

			//set materials
			s->SetMaterial("sky","mat_sky");
			s->SetMaterial("sea","mat_sea");
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
		total_verts = 250000;		//total scene vertices
	}
	else if(type == VERT_DISPLACE)
  {
	  testname = "vert_displace";
		weight = 0.30f;
    splits = 4;
		s->SetLoadList(31);	//loaded items count (must be determined manually :(

		 //camera and light position
		s->MoveCamera(0.0,-8.0,-88.0);
		s->RotateCamera(10.0,A_X);

		try
		{
			//add and position objects
			s->AddObject("tree_top","data/obj/tree_top.3ds");
			s->AddObject("tree_bark","data/obj/tree_bark.3ds");
			s->AddObject("terrain",STRIP_PLANE,300.0,300.0,200,200);
			s->MoveObj("terrain",-30.0,-0.5,0.0);
			s->AddObject("sky",SPHERE,200.0,0.0,8,8);
			s->AddObject("ground",STRIP_PLANE,50.0,100.0,500,1000);

			//add lights
			s->AddLight(0,grey,white,white,TVector(25.0,65.0,125.0),10000.0);

			//add materials & textures
			s->AddMaterial("mat_tree_top",black,white,white,64.0,0.0,0.0,GOURAUD);
			s->AddMaterial("mat_tree_bark",black,white,white,64.0,0.0,0.0,GOURAUD);
			s->AddMaterial("mat_ground",black,white,silver,128.0,0.0,0.0,GOURAUD);
			s->AddMaterial("mat_terrain",black,white,grey,64.0,0.0,0.0,GOURAUD);
			s->AddMaterial("mat_sky",black,white,white,0.0,0.0,0.0,NONE);

			//materials without displacement
			s->AddMaterial("mat_tree_doff",black,white,white,64.0,0.0,0.0,GOURAUD);
			s->AddMaterial("mat_ground_doff",black,white,silver,128.0,0.0,0.0,GOURAUD);
			s->AddMaterial("mat_terrain_doff",black,white,grey,64.0,0.0,0.0,GOURAUD);

			//cubemaps for sky
			const char *cubemaps[] = { "data/tex/cubemaps/sky/posx.tga", "data/tex/cubemaps/sky/negx.tga", "data/tex/cubemaps/sky/posy.tga",
																 "data/tex/cubemaps/sky/negy.tga", "data/tex/cubemaps/sky/posz.tga", "data/tex/cubemaps/sky/negz.tga" };
			s->AddTexture("mat_sky",cubemaps);

			//displacement material textures
			s->AddTexture("mat_tree_top","data/tex/tree.tga",BASE,MODULATE,1.0,2.0,4.0);
			s->AddTexture("mat_tree_top","data/tex/normal/tree.tga",BUMP, MODULATE, 1.0, 2.0, 4.0);
			s->AddTexture("mat_tree_top","data/tex/depth/tree.tga",DISPLACE,MODULATE,0.1f, 1.0f, 1.0f);

			s->AddTexture("mat_tree_bark","data/tex/tree.tga",BASE,MODULATE,1.0,2.0,4.0);
			s->AddTexture("mat_tree_bark","data/tex/normal/tree.tga",BUMP, MODULATE, 1.0, 2.0, 4.0);
			s->AddTexture("mat_tree_bark","data/tex/depth/tree.tga",DISPLACE,MODULATE,0.3f, 0.25f, 0.25f);

			s->AddTexture("mat_ground","data/tex/dry.tga",BASE,MODULATE,1.0,2.0,4.0);
			s->AddTexture("mat_ground","data/tex/normal/dry.tga",BUMP, MODULATE, 1.0, 2.0,4.0);
			s->AddTexture("mat_ground","data/tex/depth/dry.tga",DISPLACE,MODULATE, 2.0,2.0,4.0);
			s->AddTexture("mat_ground","data/tex/alpha/tree_shadow.tga",BASE,MODULATE);

			s->AddTexture("mat_terrain","data/tex/dust.tga",BASE,MODULATE,1.0,20.0,20.0);
			s->AddTexture("mat_terrain","data/tex/normal/terrain.tga",BUMP);
			s->AddTexture("mat_terrain","data/tex/depth/terrain.tga",DISPLACE,MODULATE, 75.0);

			//materials without displacement
			s->AddTexture("mat_tree_doff","data/tex/tree.tga",BASE,MODULATE,1.0,2.0,4.0);
			s->AddTexture("mat_ground_doff","data/tex/dry.tga",BASE,MODULATE,1.0,2.0,4.0);
			s->AddTexture("mat_ground_doff","data/tex/alpha/tree_shadow.tga",BASE,MODULATE);
			s->AddTexture("mat_terrain_doff","data/tex/dust.tga",BASE,MODULATE,1.0,20.0,20.0);

			//modified shader with static shadow map
			s->CustomShader("mat_ground","data/shaders/displaced_ground.vert",NULL,"data/shaders/displaced_ground.frag");
			s->CustomShader("mat_ground_doff","data/shaders/displaced_ground_doff.vert",NULL,"data/shaders/displaced_ground_doff.frag");

			//set materials
			s->SetMaterial("sky","mat_sky");
			s->SetMaterial("tree_top","mat_tree_doff");
			s->SetMaterial("tree_bark","mat_tree_doff");
			s->SetMaterial("ground","mat_ground_doff");
			s->SetMaterial("terrain","mat_terrain_doff");
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
		total_verts = 95890 + 4514 + 40000 + 500000 + 64;		//total scene vertices
	}

  //done
  if(!s->PostInit())
  {
		stopbit = true;
		throw ERR;
  }
  InitTime();
}

//**************************************************************************************************
//Test deinitialization, reset scene, delete dynamic data, save results
TestVert::~TestVert()
{
  s->Destroy();
}

//**************************************************************************************************
//Test run
void TestVert::Run()
{
  Test::Run();
  if(!draw_scene) return;   //don't draw scene if has ended or splashscreen is present

  /////////////////////////////////////////////////////////////////////////////////////////////////
  //switch according to test type
  if(type == VERT_LIGHTS)
  {
    if(splitbit)   //split bit set, save partial results and move to next light setup
    {
      splitbit = false;

      //set lights for actual partial scene
      switch(curr_setup)
      {
        //1 light
        case 1:
            msg = "1 light";
            break;
        //4 light
        case 2:
            msg = "4 lights";
            s->AddLight(1,black,red,red,lpos2);
            s->AddLight(2,black,green,green,lpos3);
            s->AddLight(3,black,blue,blue,lpos4);
            break;
        //8 lights
        case 3:
            msg = "8 lights";
            s->AddLight(4,black,cyan,cyan,lpos5);
            s->AddLight(5,black,magenta,magenta,lpos6);
            s->AddLight(6,black,yellow,yellow,lpos7);
            s->AddLight(7,black,silver,silver,lpos8);
            break;
        default: break;
      }
			s->UpdateLightCount();
    }
		//light movement
		s->MoveLight(0,lpos1);
	  if(curr_setup > 1)
    {
      s->MoveLight(1,lpos2);
      s->MoveLight(2,lpos3);
      s->MoveLight(3,lpos4);
    }
    if(curr_setup > 2)
    {
      s->MoveLight(4,lpos5);
      s->MoveLight(5,lpos6);
      s->MoveLight(6,lpos7);
      s->MoveLight(7,lpos8);
    }

		s->MoveCameraAbs(0.0f,-1.0f,-20.0f - anim/4.0f);
    s->Redraw();
    //info about light count
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
	else if(type == VERT_WAVES)
  {
		s->RotateCameraAbs(anim*9.0f,A_Y);
		s->SetUniform("mat_sea","time",anim);	//update animation
    s->Redraw();
	}
  else if(type == VERT_DISPLACE)
  {
    if(splitbit)   //split bit set, save partial results and move to next light setup
    {
      splitbit = false;

      //enable displacement mapping
      switch(curr_setup)
      {
        case 1:
					msg = "Displacement OFF";
					break;
				case 2:
					msg = "Displacement ON";
					s->SetMaterial("tree_top","mat_tree_top");
					s->SetMaterial("tree_bark","mat_tree_bark");
					s->SetMaterial("ground","mat_ground");
					s->SetMaterial("terrain","mat_terrain");
					break;
				default: break;
			}
		}

		//camera movement
		if(curr_setup == 1)
			s->MoveCameraAbs(-1.4f,-8.0f,-90.0f + 8.5f*anim);
		else
			s->RotateCamera(10.0f/fps,A_Y);

		s->Redraw();
		s->DrawScreenText(msg.c_str(),50,95,1.2f);
	}
	msg2 = num2str((total_verts * fps)/1000000.0f) + " MVertices/s";
	s->DrawScreenText(msg2.c_str(),90.0,1.0,1.0f);
}

//*************************************************************************************************
//Scene animation (every 10ms)
void TestVert::Animate()
{
  Test::Animate();
	switch(type)
	{
	case VERT_LIGHTS:
		//s->MoveCameraAbs(0.0,-1.0,-50.0 - anim);
    //light movement
    lpos1.x = 10.0f*cos(anim);
    lpos1.z = 45.0f*sin(anim);

    if(curr_setup > 1)
    {
      lpos2.y = 10.0f*cos(1.5f*anim) + 10.0f;
      lpos2.z = 45.0f*sin(1.5f*anim);

      lpos3.x = 10.0f*sin(anim) - 10.0f;
      lpos3.y = 10.0f*cos(0.5f*anim) + 10.0f;
      lpos3.z = 45.0f*cos(0.5f*anim);

      lpos4.x = 10.0f*sin(0.4f*anim) + 10.0f;
      lpos4.y = 10.0f*sin(0.7f*anim) + 10.0f;
      lpos4.z = 45.0f*sin(0.7f*anim);
    }
    if(curr_setup > 2)
    {
      lpos5.x = 20.0f*cos(1.3f*anim);
      lpos5.z = 20.0f*sin(1.3f*anim);

      lpos6.y = 10.0f*cos(1.8f*anim) + 10.0f;
      lpos6.z = 10.0f*sin(1.8f*anim);

      lpos7.x = 10.0f*sin(1.6f*anim) - 10.0f;
      lpos7.y = 10.0f*cos(1.6f*anim) + 10.0f;
      lpos7.z = 10.0f*cos(1.6f*anim);

      lpos8.x = 10.0f*sin(0.9f*anim) + 10.0f;
      lpos8.y = 10.0f*sin(0.8f*anim) + 10.0f;
      lpos8.z = 10.0f*sin(0.8f*anim);
    }
		break;
	case VERT_WAVES:
		//s->RotateCameraAbs(anim*10.0,A_Y);
		break;
	case VERT_DISPLACE:
		/*
		//camera movement
		if(curr_setup == 1)
			s->MoveCameraAbs(-1.4,-8.0,-90.0 + 7.0*anim);
		else
			s->RotateCamera(0.2,A_Y);
		*/
		break;
	}
}

//**************************************************************************************************
//Save results to XML file
void TestVert::SaveResults()
{
  Test::SaveResults();

	vert_rate = (total_verts * avg_fps)/1000000.0f;
  //switch according to test type
  if(type == VERT_LIGHTS)
	{
    //test name
    switch(curr_setup)
    {
      case 1:
          results<<"<vertex>\n"
                  "<"<<testname<<">\n"
                  "<light1>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<vert_rate<<"</vert_rate>\n</light1>\n";
					test_score += avg_fps*5;
          break;
      case 2:
          results<<"<light4>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<vert_rate<<"</vert_rate>\n</light4>\n";
					test_score += avg_fps*5;
          break;
      case 3:
					test_score += avg_fps*7;
					//final weight
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<light8>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<vert_rate<<"</vert_rate>\n</light8>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == VERT_DISPLACE)
  {
    //test name
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"
                  "<disp_off>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<vert_rate<<"</vert_rate>\n</disp_off>\n";
          break;
      case 4:
					//final weight
					test_score = avg_fps * 2;
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<disp_on>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<vert_rate<<"</vert_rate>\n</disp_on>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n"
							"</vertex>";
          break;
      default: break;
    }
  }
	//else write only test name and FPS
  else
  {
		//final weight
		test_score += vert_rate * 5.5f;
		cout<<"SCORE: "<<test_score<<endl;
    results<<"<"<<testname<<">\n"
           "<fps>"<<avg_fps<<"</fps><vert_rate>"<<vert_rate<<"</vert_rate>\n\n"
					 "<score>"<<test_score<<"</score>"
           "</"<<testname<<">\n";
  }
}
